@@ -128,16 +128,17 @@ class Traits:
         ## Traits endpoint for create is old demdex URL
         request_url = "https://api.demdex.com/v1/traits/"
         ## Required columns for API call
-        reqd_cols = pd.DataFrame(columns=['traitType', 'name', 'dataSourceId', 'folderId', 'traitRule', 'ttl', 'description', 'comments'])
+        reqd_cols = pd.DataFrame(columns=['traitType', 'name', 'dataSourceId', 'folderId', 'traitRule'])
         ## Load csv into pandas df
         if file_path.endswith('.csv'):
             df = pd.read_csv(file_path, engine='python')
         else:
             raise Exception('File type is not csv.')
         ## Check for reqd cols
-        if list(df.columns) != list(reqd_cols.columns):
+        col_check = all(item in reqd_cols.columns for item in df.columns)
+        if not col_check:
             reqd_cols.to_csv('aam_trait_create_template.csv', index=False)
-            raise Exception('Column names are incorrect. Please re-upload file with template.')
+            raise Exception('Missing one or more required columns. Please re-upload file with template.')
         traits_as_dict = df.to_dict(orient='records')
         
         ## Declare counter vars
@@ -145,7 +146,7 @@ class Traits:
         num_successful_traits = 0
         
         ## Handle for bad traits
-        unsuccessful_traits = pd.DataFrame(columns=['traitType', 'name', 'dataSourceId', 'folderId', 'traitRule', 'ttl', 'description', 'comments'])
+        unsuccessful_traits = pd.DataFrame(columns=df.columns)
                
         for trait in traits_as_dict:
             trait_json = json.dumps(trait)
@@ -177,7 +178,8 @@ class Traits:
         else:
             raise Exception('File type is not csv.')
         ## Check for reqd cols
-        if list(df.columns) != list(reqd_cols.columns):
+        col_check = all(item in reqd_cols.columns for item in df.columns)
+        if not col_check:
             reqd_cols.to_csv('aam_trait_delete_template.csv', index=False)
             raise Exception('Column name should be sid. Please re-upload file with template.')
         
@@ -228,16 +230,17 @@ class Traits:
     @classmethod
     def update_many(cls, file_path):
         ## Required columns for API call
-        reqd_cols = pd.DataFrame(columns=['traitType', 'name', 'dataSourceId', 'folderId', 'sid'])
+        reqd_cols = pd.DataFrame(columns=['sid'])
         ## Load csv into pandas df
         if file_path.endswith('.csv'):
             df = pd.read_csv(file_path, engine='python')
         else:
             raise Exception('File type is not csv.')
         ## Check for reqd cols
-        if list(reqd_cols.columns) != list(df.columns):
+        col_check = all(item in df.columns for item in reqd_cols.columns)
+        if not col_check:
             reqd_cols.to_csv('aam_trait_update_template.csv', index=False)
-            raise Exception('CSV should include at least the following columns, with additional columns you want to update. Please re-upload file using template.')
+            raise Exception('Missing sid columns. Please re-upload file with template.')
         
         ## Declare counter vars
         num_traits_in_file = len(df)
@@ -246,22 +249,32 @@ class Traits:
         ## Handle for bad traits
         unsuccessful_traits = pd.DataFrame()
         
-        ## Get request data
-        traits_as_dict = df.to_dict(orient='records')
-
-        for trait in traits_as_dict:
-            sid = trait['sid']
-            trait_json = json.dumps(trait)  
-            request_url = "https://api.demdex.com/v1/traits/{0}".format(sid)
-            response = requests.put(url = request_url,
-                                    headers = Headers.createHeaders(json=True),
-                                    data = trait_json)
-            ## Print error code if get request is unsuccessful
-            if response.status_code != 200:
-                print("Attempt to update trait {0} was unsuccessful. \nError code {1}. \nReason: {2}".format(sid, response.status_code, response.content.decode('utf-8')))
-                unsuccessful_traits = unsuccessful_traits.append(trait, ignore_index=True)
-            else:
-                num_successful_traits += 1
+        for index, row in df.iterrows():
+            ## Get current trait info
+            try:
+              current_trait_info = Traits.get_one(sid=row['sid'])
+              current_trait_info = current_trait_info[['traitType', 'name', 'dataSourceId', 'folderId', 'sid']]
+              ## Determine diff of given file and trait GET request
+              col_diff = list(current_trait_info.columns.difference(df.columns))
+              current_trait_info = current_trait_info[col_diff]
+              current_trait_info['sid'] = row['sid']
+              trait_to_update = row.to_frame().T
+              ## Merge required cols if they do not exist in CSV
+              updated_trait = pd.merge(trait_to_update, current_trait_info)
+              ## Get request data
+              trait_as_dict = updated_trait.to_dict(orient='records')[0]
+              sid = trait_as_dict['sid']
+              trait_json = json.dumps(trait_as_dict)  
+              request_url = "https://api.demdex.com/v1/traits/{0}".format(sid)
+              response = requests.put(url = request_url,
+                                      headers = Headers.createHeaders(json=True),
+                                      data = trait_json)
+              if response.status_code == 200:
+                num_successful_traits += 1       
+            except:
+                print(f'Attempt to update trait {row["sid"]} was unsuccessful.')
+                unsuccessful_traits = unsuccessful_traits.append(row, ignore_index=True)
+                
  
         ## Return bad traits
         if len(unsuccessful_traits) > 0:
